/*
* Recursive descent parser for simple C expressions.
* Works only on single digit integers
*/
/*
Example data:

1
1+2
1+2 * 3+4
1+--4
1+(2*3)+4

*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

int evalExp(void);
int evalMultiplicativeExp(void);
int evalUnaryExp(void);
int getPrimaryExp(void);

int skipSpace()
{
  int chRead;

  do
  {
    chRead = getchar();
  }
  while (chRead==' ' || chRead == '\t');
  return chRead;
}


int evalExp(void)
{
    int val, chRead;

    val = evalMultiplicativeExp();
    for (;;)
    {
        chRead = skipSpace();
        // fprintf(stderr, "evalExp: '%c'\n", chRead);

        switch (chRead)
        {
        case '+':
            val = val + evalMultiplicativeExp();
            break;
        case '-':
            val = val - evalMultiplicativeExp();
            break;
        default:
            ungetc(chRead, stdin);
            // fprintf(stderr, "evalExp ungetc: '%c'\n", chRead);
            return(val);
        }
    }
}

int evalMultiplicativeExp(void)
{
    int val, chRead;

    val = evalUnaryExp();
    for (;;)
    {
        chRead = skipSpace();
        // fprintf(stderr, "evalMultiplicativeExp: '%c'\n", chRead);
        switch (chRead)
        {
        case '*':
            val = val * evalUnaryExp();
            break;
        case '/':
            val = val / evalUnaryExp();
            break;
        case '%':
            val = val % evalUnaryExp();
            break;
        default:
            // fprintf(stderr, "evalMultiplicativeExp ungetc: '%c'\n", chRead);
            ungetc(chRead, stdin);
            return(val);
        }
    }
}

int evalUnaryExp(void)
{
    int val, chRead;

    chRead = skipSpace();
    // fprintf(stderr, "evalUnaryExp: '%c'\n", chRead);
    switch (chRead)
    {
    case '+':
        val = evalUnaryExp();
        break;
    case '-':
        val = -evalUnaryExp();
        break;
    default:
        ungetc(chRead, stdin);
        // fprintf(stderr, "evalUnaryExp ungetc: '%c'\n", chRead);
        val = getPrimaryExp();
        break;
    }
    return(val);
}

int getPrimaryExp(void)
{
    int val, chRead;

    chRead = skipSpace();
    // fprintf(stderr, "getPrimaryExp: '%c'\n", chRead);
    switch(chRead)
    {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        val = chRead - '0';
        break;
      case '(':
        val = evalExp();
        chRead = skipSpace();
        if (chRead != ')')
        {
          // fprintf(stderr, "Error: getPrimaryExp no closing ')' - read %c\n", chRead);      /* skip closing ')' */
          exit(EXIT_FAILURE);
        }
        break;
      default:
        printf("Error: getPrimaryExp invalid character read %c\n", chRead);
        exit(EXIT_FAILURE);
    }
    return(val);
}


int main()
{
    int val, chRead;

    for (;;)
    {
        printf("Expression: ");
        val = evalExp();
        if ((chRead = getchar()) != '\n')
        {
            // fprintf(stderr, "Error: incorrect expression at %c\n", chRead);
            while (getchar() != '\n') // skip rest
                ; /* NULL */
        }
        else
        {
            printf("Value of expression is %d\n", val);
        }
    }
    return 0;
}
